/*
 * Attribute.java
 *
 * Created on May 24, 2005, 11:09 PM
 */

/*
 * Attribute.java
 *
 * Created on February 22, 2004, 11:11 PM
 */

package datamining;

/**
 *
 * @author  mike
 */
public class Attribute {
    
    private String name = "unknown";
    
    private String[] possibleValues;
    
    /** Creates a new instance of Attribute  - use a default size */
    public Attribute() {        
        possibleValues = new String[10];
    }
    
    /** Creates a new instance of Attribute 
     * given name and size - create the array but don't fill it 
     */
    public Attribute(String nam, int size) {
        name = nam;
        possibleValues = new String[size];
    }
    
    /** Creates a new instance of Attribute 
     * given name and array of strings as possible values - create the array and fill it 
     */
    public Attribute(String nam, String [] possVals) {
        name = nam;
        possibleValues = new String[possVals.length];
        for (int i = 0; i < possVals.length; i++){ 
            possibleValues[i] = possVals[i];
        }
    }
    
    /** Creates a new instance of Attribute 
     * given line from ARFF file - create the array and fill it 
     */
    public Attribute(String line) {
        if ( ! line.startsWith("@attribute") ) {
            System.out.println("Serious error creating attribute info");
        }
        else {
            String part = line.substring(10); // strip off @attribute tag
            int curly = part.indexOf('{');  // find curly starting possible values
            // if attribute is numeric cutoff - don't use - can tell because curly brace is not found
            if (curly < 0) {
                name = "numeric";
            }
            else {
                name = part.substring(0,curly-1).trim(); // extract attribute name, trimming blanks
                int endCurly = part.indexOf('}');
                String valArea = part.substring(curly+1,endCurly); // pull out substring between curlys
                System.out.println("Debug - valArea: " + valArea);
                // find out how many possible values
                int size = 1; // always should have at least one value - look for commas to indicate more
                int commaPos = 0; 
                int lookFrom = 0; // first place to look
                while (commaPos >= 0) {
                    commaPos = valArea.indexOf(',',lookFrom);
                    if (commaPos >= 0) { 
                        // found one
                        size++;
                        lookFrom = commaPos+1; // look after this
                    }
                }
                // now now the size of array to create
                possibleValues = new String[size];
                // fill the array
                int pos = 0;
                String latest = "";
                for (int i = 0; i < size; i++){ 
                    while ((pos < valArea.length()) && (valArea.charAt(pos) != ',')) {
                        latest += valArea.charAt(pos);
                        pos++;
                    }
                    possibleValues[i] = latest.trim();
                    latest = ""; // reset to blank
                    pos++;  // get past the comma
                }
            }
        }
    }
    
    /** 
     * report name of the attribute
     */
    public String getName() {
        return name;
    }
    
    
    /** 
     * reports whether given string is one of the possible values for this attribute
     * 
     */
    public boolean isPossVal(String q) {
        // defautl to not found
        boolean res = false;
        for (int i= 0; i < possibleValues.length; i++) {
            if (q.equals(possibleValues[i])) {
                // found it
                res = true;
            }
        }
        return res;
    }
    
    
    /** 
     * reports location of given string within the possible values for this attribute
     * This method will return the last place it was found - hopefully it is only in there once
     */
    public int whichPossVal(String q) {
        // default to not found
        int res = -1;
        for (int i= 0; i < possibleValues.length; i++) {
            if (q.equals(possibleValues[i]) ) {
                // found it
                res = i;
            }
        }
        return res;
    }
    
    /** 
     * reports possible value for a given location within the possible values for this attribute
     * 
     */
    public String getNthPossVal(int n) {
        return possibleValues[n];
    }
    
    
    /** 
     * reports how many possible values for this attribute - give real values - not space in the array
     */
    public int getHowManyPossVal() {
        // 
        int res = 0;
        for (int i= 0; i < possibleValues.length; i++) {
            if (possibleValues[i] != null) {
                // really there
                res++;
            }
        }
        return res;
    }
    
    
    /** 
     * give a String representation of the object
     */
    public String toString () {
        String res = "@attribute " + getName() + " { ";
        for (int i= 0; i < possibleValues.length; i++) {
            res += " " + possibleValues[i] + " ";
        }
        res += " } ";
        return res;
    }
        
    
    /////////////////////////////////////////////////////////////////////////
    /// mutators
    /////////////////////////////////////////////////////////////////////////
    
    // could use ways to add to list of possible values
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        System.exit(0);
    }
    
}

