/*
 * CrossValidation.java
 *
 * Created on May 24, 2005, 11:13 PM
 */

/*
 * CrossValidation.java
 *
 * Created on February 23, 2004, 11:45 AM
 */

package datamining;


import java.util.Vector;
import java.util.Random;

/**
 *
 * @author  redmond
 */
public class CrossValidation {
    
    /** Creates a new instance of CrossValidation */
    public CrossValidation() {
    }
    
    /**
     * pick out instances to use in each of 10 folds
     * Simplest possible assignment - divide in order
     * Really too simple - unless the instances are going to be in Random order
     * - returns Vector of Vectors - one Vector of instance numbers for each fold
     */
    public static Vector pickFoldInstances (int numInstances) {
        Vector res = pickFoldInstances (numInstances,10);
        return res;
    }
    
    /**
     * pick out instances to use in each of 10 folds
     * Random assignment - returns Vector of Vectors - one Vector of instance numbers for each fold
     */
    public static Vector pickRandomFoldInstances (int numInstances) {
        Vector res = pickRandomFoldInstances (numInstances,10);
        return res;
    }
    
    
    /**
     * pick out instances to use in each of 10 folds
     * Stratefied Random assignment - 
     *    tries to spread classes equally among folds
     * returns Vector of Vectors - one Vector of INSTANCES for each fold
     *     (since passing instances - cut out the middle man, separate instance instead 
     *       of just identifying them)
     * NOT IMPLEMENTED YET!!
     */
    public static Vector pickStratefiedRandomFoldInstances (Vector allInstances) {
        Vector res = pickStratefiedRandomFoldInstances (allInstances,10);
        return res;
    }
    
    
    /**
     * pick out instances to use in each of numFolds folds
     * Simplest possible assignment - divide in order
     * Really too simple - unless the instances are going to be in Random order
     * - returns Vector of Vectors - one Vector of instance numbers for each fold
     */
    public static Vector pickFoldInstances (int numInstances, int numFolds) {
        Vector listOfLists = new Vector(numFolds);
        int numPer = numInstances / numFolds;
        int leftover = numInstances % numFolds; // the first leftover folds will have an extra instance
        // loop through all folds - setting up list of instances in each fold
        int inst = 0;
        for (int fold = 0; fold < numFolds; fold++) {
            Vector foldList;
            int numInCurrFold = numPer;
            // create vector for this fold - with appropriate number of elements
            if (fold < leftover) {
                numInCurrFold++;
            }                
            foldList = new Vector(numInCurrFold);
            // fill vector for current fold
            for (int cnt = 0; cnt < numInCurrFold; cnt++) {
                Integer toAdd = new Integer(inst);
                foldList.add( toAdd);  // instance number to be added to fold
                inst++;
            }
            // have created vector for current fold - add to the overall vector
            listOfLists.add(foldList);
        }
        return listOfLists;            
    }
    
    /**
     * pick out instances to use in each of numFolds folds
     * Random assignment - returns Vector of Vectors - one Vector of instance numbers for each fold
     */
    public static Vector pickRandomFoldInstances (int numInstances, int numFolds) {
        // set up random number generator
        Random generator = new Random();
        // create a vector containing all possible instances numbers - from which we 
        // will pick instances to include in folds
        Vector toPickFrom = new Vector(numInstances);
        for (int i = 0; i < numInstances; i++) {
            Integer wrapped = new Integer(i);
            toPickFrom.add (wrapped);
        }
        Vector listOfLists = new Vector(numFolds);
        int numPer = numInstances / numFolds;
        int leftover = numInstances % numFolds; // the first leftover folds will have an extra instance
        // loop through all folds - setting up list of instances in each fold
        for (int fold = 0; fold < numFolds; fold++) {
            Vector foldList;
            int numInCurrFold = numPer;
            // create vector for this fold - with appropriate number of elements
            if (fold < leftover) {
                numInCurrFold++;
            }                
            foldList = new Vector(numInCurrFold);
            // fill vector for current fold
            for (int cnt = 0; cnt < numInCurrFold; cnt++) {
                // randomly pick a number in the list
                int next = generator.nextInt(toPickFrom.size());
                Integer toAdd = (Integer)toPickFrom.get(next);
                foldList.add( toAdd);  // instance number to be added to fold
                // now remove from the list to pick from so we don't get repeats
                toPickFrom.removeElementAt(next);                
            }
            // have created vector for current fold - add to the overall vector
            listOfLists.add(foldList);
        }
        return listOfLists;
    }
    
    
    /**
     * pick out instances to use in each of numFolds folds
     * Stratefied Random assignment - 
     *    tries to spread classes equally among folds
     * returns Vector of Vectors - one Vector of INSTANCES for each fold
     *     (since passing instances - cut out the middle man, separate instance instead 
     *       of just identifying them)
     * NOT IMPLEMENTED YET!!
     */
    public static Vector pickStratefiedRandomFoldInstances (Vector allInstances, int numFolds) {
        Vector listOfLists = new Vector(numFolds);
        return listOfLists;
    }
    
     
    /** 
     * merge confusion matrices, returning the sum
     * assumes that the passed matrices are the same dimensions
     */
    
    public static int[][] mergeConfusion (int [] [] summary, int [][] newc) {
        int size = summary.length;
        int [][] result = new int[size][size];
        for (int i = 0; i < size; i++ ) {
            for (int j = 0; j < size; j++) {
                result[i][j] = summary[i][j] + newc[i][j];
            }
        }
        return result;
    }
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        System.exit(0);
    }
    
}

