/*
 * PowerUnitProblemSpecificInfo.java
 *
 * Created on July 14, 2005, 11:16 AM
 */

/*
 * PowerUnitProblemSpecificInfo.java
 *
 * Created on July 7, 2005, 12:54 AM
 */

package sgapowerunit;


import java.io.File;
import java.io.*;
import java.util.Vector;
import datamining.*;

/**
 *
 * @author Mike
 */
public class PowerUnitProblemSpecificInfo extends ProblemSpecificInfo {
    
    private Vector powerUnitList;
    private int maxExpectedLoads[];
    
    /** Creates a new instance of PowerUnitProblemSpecificInfo */
    public PowerUnitProblemSpecificInfo() {
        powerUnitList = null;
        maxExpectedLoads = new int[4];
    }
    
    /** Creates a new instance of PowerUnitProblemSpecificInfo */
    public PowerUnitProblemSpecificInfo(Vector powerUnits, int expectedLoads[]) {
        powerUnitList = powerUnits;
    
        maxExpectedLoads = expectedLoads;
        
    }
    
    /** Creates a new instance of PowerUnitProblemSpecificInfo from a buffered reader.
     *  Format of file is expected to be:
     *  numberofUnits
     *  a line for each unit - each with unit number, capacity, number of periods for maintanance
     *  a line for each quarter - each with maximum expected load for that quarter.
     *  Assumes 4 quarters
     */
    public PowerUnitProblemSpecificInfo(BufferedReader bReader) throws IOException {
        
        String line;
        line = bReader.readLine();
        int numUnits = Integer.parseInt(line);   // if this bombs, fix the file
        powerUnitList = new Vector(numUnits);
        // loop through all units - geting info for each
        for (int unit = 0; unit < numUnits; unit++) {
            line = bReader.readLine();
            int commaPos = 0; 
            int lookFrom = 0; // first place to look
            // get unit number
            commaPos = line.indexOf(',', lookFrom);
            String strUnitNum = line.substring(0, commaPos);   
            int unitNum = Integer.parseInt(strUnitNum);                        
            lookFrom = commaPos+1; // look after this
            // get unit capacity
            commaPos = line.indexOf(',', lookFrom);
            String strCapac = line.substring(lookFrom, commaPos);  
            int capacity = Integer.parseInt(strCapac);                      
            lookFrom = commaPos+1; // look after this
            // get unit number of intervals
            String strNumInterv = line.substring(lookFrom);
            int numInterv = Integer.parseInt(strNumInterv);
            // create power unit and add to vector
            PowerUnit toAdd = new PowerUnit(unitNum, capacity,numInterv); 
            powerUnitList.add(toAdd);
        }
        // now work on max expected loads
        maxExpectedLoads = new int[4];
        for (int qtr = 0; qtr < 4; qtr++) {
            line = bReader.readLine();
            maxExpectedLoads[qtr] = Integer.parseInt(line);   // if this bombs, fix the file
        }        
        bReader.close();
    }
    
    public Vector getPowerUnitList () {
        return powerUnitList;
    }
    
    public int[] getMaxExpectedLoads () {
        return maxExpectedLoads;
    }
    
    public int getNthMaxExpectedLoad (int N) {
        if (N < 4) {
            return maxExpectedLoads[N];
        }
        else {
            System.out.println("Program error - expected load doesn't exist");
            System.exit(0);
        }
        // will never really get here
        return -9;
    }
    
    public void setPowerUnitList (Vector powerUnits) {
        powerUnitList = powerUnits;
    }
    
    /// capability to add to the vector?
    
    public void setMaxExpectedLoads (int loads[]) {
        maxExpectedLoads = loads;
    }
    
    public boolean setNthMaxExpectedLoad (int N, int load) {
        if ((N < 4) || (load < 0) ) {
            return false;
        }
        else {
            maxExpectedLoads[N] = load;
            return true;
        }
    }
    
    
    /**
     * build a Vector of attributes from the power unit list part of the problem specific info 
     */
    public Vector buildAttributes ( )  {
        String[][] values = { {"firstQtr", "secondQtr", "thirdQtr", "fourthQtr" } ,         // possible schedule if maint takes 1 qtr
        { "first2", "middle2", "last2" } ,                                                  // possible schedule if maint takes 2 qtrs
        { "first3", "last3" } };                                                            // possible schedule if maint takes 3 qtrs
                                                                                            // a unit that toke 4 qtrs would be worthless
        
          
        Vector attributes = new Vector(powerUnitList.size());
        String line;
        // get all attribute info - go though whole powerUnitList
        for (int unit = 0; unit < powerUnitList.size(); unit++){ 
            PowerUnit source = (PowerUnit)powerUnitList.get(unit);
            // in creating the attribute, the key thing is how many intervals are needed for maintenance
            int numInterv = source.getNumIntervalsForMaint();
            String nam = "unit" + (unit+1);
            String[] attrVals = values[numInterv-1]; // number of intervals 1-3 - indexes into the 2D array are 0-2
            // create the attribute
            Attribute latest = new Attribute(nam, attrVals);
            attributes.add(latest); // add to vector of attributes
            System.out.println("Attribute created: " + latest);
        }
        return attributes;
    }
    
    public String toString () {
        String res = "";
        res += powerUnitList.size() + "\n";
        // get all attribute info - go though whole powerUnitList
        for (int unit = 0; unit < powerUnitList.size(); unit++){ 
            PowerUnit source = (PowerUnit)powerUnitList.get(unit);
            String unitString = source.toString() + "\n";
            res += unitString;
        }
        for (int qtr = 0; qtr < 4; qtr++) {
            res += maxExpectedLoads[qtr] + "\n";
        }
        return res;
    }
    
    
}