/*
 * PowerUnitSolution.java
 *
 * Created on July 14, 2005, 11:25 AM
 */
/*
 * PowerUnitSolution.java
 * Used for aspects of solution that are specific to the PowerUnit domain
 *
 * Created on July 6, 2005, 2:48 PM
 */

package sgapowerunit;


import java.util.Vector;

/**
 *
 * @author Mike
 */
public class PowerUnitSolution extends Solution implements Ratable { 
    
    //private Solution toRate;  // to allow stashing so rate methods can get to strength and change it
    
    /** Creates a new instance of PowerUnitSolution */
    public PowerUnitSolution() {
    }
    
    /** Creates a new instance of PowerUnitSolution based on a more general solution */
    public PowerUnitSolution(Solution toCreate) {
        super("dummy"); // fake out so that names don't skip numbers
        this.setName(toCreate.getName());
        this.setPattern(toCreate.getPattern());
        this.setStrength(toCreate.getStrength());
        //toRate = toCreate;
    }
    
    /**
     * static method used to determine if a given service string indicates that a unit is being serviced in a given quarter
     * Careful! - quarters are 0-3
     */
    public static boolean beingServiced (String serv, int qtr) {
        switch (qtr) {
            case 0:
                    if (serv.equals("firstQtr")
                        || serv.equals("first2")
                        || serv.equals("first3")) {
                        return true;
                    }
                    else {
                   break;
                    }
            case 1:
                    if (serv.equals("secondQtr")
                        || serv.equals("first2")
                        || serv.equals("middle2")
                        || serv.equals("last3")
                        || serv.equals("first3")) {
                        return true;
                    }
                    else {
                   break;
                    }
            case 2:
                    if (serv.equals("thirdQtr")
                        || serv.equals("last2")
                        || serv.equals("middle2")
                        || serv.equals("last3")
                        || serv.equals("first3")) {
                        return true;
                    }
                    else {
                   break;
                    }
            case 3:
                    if (serv.equals("fourthQtr")
                        || serv.equals("last2")
                        || serv.equals("last3")) {
                        return true;
                    }
                    else {
                   break;
                    }
        }
        // if got here - must not match
        return false;
        } 
    
    
    
    /**
     * Evaluate a solution based on how well it meets the problem requirements. 
     * Also, besides returning the rating, set it as the strength of the rule.
     * For all quarters, find capacity of units not being serviced, compare to expected load requirements - and find excess capacity
     * Rating is the smallest excess capacity among all of the quarters
     *
     * need maximum expected loads for each quarter ##################3
     *  
     */
    public double rateChromosome (ProblemSpecificInfo passed) {
        // only really can handle if involves Power Unit
        if ( passed instanceof PowerUnitProblemSpecificInfo) {
            PowerUnitProblemSpecificInfo info = (PowerUnitProblemSpecificInfo) passed;
            // pull out key info
            Vector powerUnits = info.getPowerUnitList();
            int[] loads = info.getMaxExpectedLoads();
            double rating = rateSolution(powerUnits,loads);
            return rating;
        }
        else {
            // trouble
            System.out.println("Program error - cannot rate chromosome");
            System.exit(0);
        }
        // should never get here
        return -9;
        
    }
    
    
    /**
     * Evaluate a solution based on how well it meets the problem requirements. 
     * Also, besides returning the rating, set it as the strength of the rule.
     * For all quarters, find capacity of units not being serviced, compare to expected load requirements - and find excess capacity
     * Rating is the smallest excess capacity among all of the quarters
     *
     * need maximum expected loads for each quarter ##################3
     *  
     */
    public double rateSolution (Vector powerUnitList, int maxExpectedLoads[]) {
        // loop through the training data, seeing how the rule fares with each
        int lowestReserve = 1000000; // larger than it will ever be I hope to force first qtr to be lowest at first
        int powerAvail[] = new int[4]; // mostly for debugging
        int reserves[] = new int[4];
        int numPattern = this.getLength(); // number of clauses on pattern
        int numUnits = powerUnitList.size(); // number of units we have info about
        // if solution has different number of units than we have info about - we'return dead
        if (numPattern != numUnits) {
            System.out.println("Program failure- solution is not right size");
            System.exit(0);
        }
        else {
            System.out.println("     rating solution " + this); 
            int currReserve = 0;
            // loop through all quarters - assume 4
            for (int qtr = 0; qtr < 4; qtr++) {
                // loop through the solution, seeing which units are being serviced for this quarter
                int currPow = 0;
                int totalPow = 0;
                for (int patt = 0; patt < numPattern; patt++) {
                    String service = this.getNthPattern(patt);
                    if (PowerUnitSolution.beingServiced(service, qtr)) {
                        currPow = 0;
                    }
                    else {
                        // must be on-line for this quarter
                        // need to find out capacity
                        PowerUnit curr = (PowerUnit) powerUnitList.get(patt);
                        currPow = curr.getUnitCapacity();
                    }
                    totalPow += currPow;
                }                 
                powerAvail[qtr] = totalPow;
                currReserve = totalPow - maxExpectedLoads[qtr];
                reserves[qtr] = currReserve;
                // see if new low
                if (currReserve < lowestReserve) {
                    lowestReserve = currReserve;
                }
                
            } // end looping through qtrs 
        }
        
        this.setStrength(lowestReserve);
        System.out.println("     Power Capacities: " + powerAvail[0] + " | " + powerAvail[1] + " | " + powerAvail[2] + " | " + powerAvail[3] + " | " 
                + "\n          Reserves: " + reserves[0] + " | " + reserves[1] + " | " + reserves[2] + " | " + reserves[3] + " | "
                + "\n          Rating: " + this.getStrength());
        // keep up with best ever
        if (lowestReserve > highestRating) {
            highestRating = lowestReserve;
            highestRatedSolution = this;
            System.out.println("==========New Highest Rated Solution == Rating " + lowestReserve + " === Rule: " + this);
        }
        return lowestReserve;
    }
            
            
            
      
   
    
  
    
}