/*
 * SGADesignProb.java
 *
 * Created on July 14, 2005, 11:28 AM
 */
/*
 * SGADesignProb.java
 *     used for solutions to problems that are not classification problems - e.g. design, scheduling
 *      Plan on using this for PowerPlant Maint Scheduling and for Student Group assignment
 *
 * Created on July 7, 2005, 12:34 AM
 */

package sgapowerunit;


import java.util.Vector;
import java.io.File;
import java.io.*;
import datamining.*;

/**
 *
 * @author Mike
 */
public class SGADesignProb {
    
    /** Creates a new instance of SGADesignProb */
    public SGADesignProb() {
    }
    
    
    
    /**
     * determine whether some random criteria is met
     */
    public static boolean yesNo( double prob) {
        double rand = Solution.getGenerator().nextDouble();
        if (rand < prob) {
            return true;
        }
        else {
            return false;
        }
    }
    
    /** 
     * train the simple genetic algorithm method (giving trace of significant outputs)
     *      - returns a Vector of Solutions 
     *            each Solution is a Vector of attribute values plus a strength of the solution
     *               (the attribute values CANNOT include a wildcard)
     *
     */
    public static Vector Train (Vector attrDefs, int popSize, int generations, double pMutate, ProblemSpecificInfo probInfo, PrintWriter tracefile) {
                
        Vector solutionSet = new Vector();
        Solution [] solutionArr = Solution.genNRandomSolutions(attrDefs,popSize);
        solutionSet = Solution.convertArrayToVector(solutionArr);
        Solution.showSolutionSet(solutionSet);
        
        int numCross = popSize / 2;
        
        Vector res = null;
        
        
        // MAR 07/07/05 - show trace info if using tracefile
        if (tracefile != null) {
            tracefile.println("generation, high, mean");
        }
        
        double total = 0;
        // loop through all Solutions in the Solution set, rating the Solution on the current info about the problem
        for (int sol = 0; sol < solutionSet.size(); sol++) {
            Solution currSol = (Solution) solutionSet.get(sol);
            double score = 0;
            if (probInfo instanceof PowerUnitProblemSpecificInfo) {
                PowerUnitSolution curr = new PowerUnitSolution( currSol );
                score = curr.rateChromosome(probInfo);
            }
            /*
             * commented out for PowerUnit specific project
            else if (probInfo instanceof StudentProblemSpecificInfo) {
                StudentSolution curr = new StudentSolution( currSol );
                score = curr.rateChromosome(probInfo);
            }
             **/
            // make sure solution reflects the rating
            currSol.setStrength(score);
            total += score;
        }
        System.out.println("======Ratings======");   
        Solution.showSolutionSet(solutionSet);
        
        System.out.println("\n     High: " + Solution.twoDecPlacesFormat.format(Solution.getSolutionSetHighRating(solutionSet)));
        System.out.println("     Mean: " + Solution.twoDecPlacesFormat.format(Solution.getSolutionSetMeanRating(solutionSet)) + "\n");
        
        // MAR 07/05/05 - show trace info if using tracefile
        if (tracefile != null) {
            tracefile.println("0," + Solution.getSolutionSetHighRating(solutionSet) + "," + Solution.getSolutionSetMeanRating(solutionSet));
        }
        
        // loop through all generations
        for (int gen = 0; gen < generations; gen++) {
            
            // work on next generation
            Vector newSet = new Vector();
            for (int pair = 0; pair < numCross; pair++) {
                // choose parents based on strength
                Solution parent1 = Solution.chooseProportional(solutionSet, total);
                Solution parent2 = Solution.chooseProportional(solutionSet, total);
                System.out.println("\n Parents:");
                System.out.println("      " + parent1);
                System.out.println("      " + parent2);
                System.out.println("\n");
                
                int crossOverPt = Solution.getGenerator().nextInt(attrDefs.size()); 
                int len = parent1.getLength();
                Solution child1 = new Solution(len);
                Solution child2 = new Solution(len);
                Solution.crossover(parent1, parent2, crossOverPt, child1, child2);
                newSet.add(child1);
                newSet.add(child2);
            }
            
            // consider reducing probability of mutation as generations move on
            
            
            // consider mutations
            // loop through all solutions
            for (int sol = 0; sol < newSet.size(); sol++) {
                Solution curr = (Solution)newSet.get(sol);
                int len = curr.getLength();
                // loop through all attributes
                for (int at = 0; at < len; at++) {
                    boolean mutate = SGADesignProb.yesNo(pMutate);
                    if (mutate) {
                        // mutating - figure out which value to change to
                        System.out.print("   mutating solution " + sol + " attr " + at + " to ");
                        Attribute currAtt = (Attribute)attrDefs.get(at);
                        int choices = currAtt.getHowManyPossVal(); 
                        int rand = Solution.getGenerator().nextInt(choices);
                        // randomly generate new value
                        String newVal = currAtt.getNthPossVal(rand);
                        System.out.println(newVal);
                        curr.setNthpattern(at, newVal);
                            
                        }
                    }
                }
            
            // bring in the new generation
            solutionSet = newSet;
            
            
            // rate before using
            total = 0;
            // loop through all Solutions in the Solution set, rating the Solution on the current info about the problem
            for (int sol = 0; sol < solutionSet.size(); sol++) {
                //PowerUnitSolution curr = (PowerUnitSolution) solutionSet.get(sol);
                Solution currSol = (Solution) solutionSet.get(sol);
                double score = 0;
                if (probInfo instanceof PowerUnitProblemSpecificInfo) {
                    PowerUnitSolution curr = new PowerUnitSolution( currSol );
                    score = curr.rateChromosome(probInfo);
                }
            /*
             * commented out for PowerUnit specific project
                else if (probInfo instanceof StudentProblemSpecificInfo) {
                    StudentSolution curr = new StudentSolution( currSol );
                    score = curr.rateChromosome(probInfo);
                }
             **/
                // make sure solution reflects the rating
                currSol.setStrength(score);
                total += score;
            }
            System.out.println("======Ratings======");   
            Solution.showSolutionSet(solutionSet);

            System.out.println("\n     High: " + Solution.twoDecPlacesFormat.format(Solution.getSolutionSetHighRating(solutionSet)));
            System.out.println("     Mean: " + Solution.twoDecPlacesFormat.format(Solution.getSolutionSetMeanRating(solutionSet)) + "\n");

            // MAR 07/05/05 - show trace info if using tracefile
            if (tracefile != null) {
                tracefile.println((gen+1) + ","  + Solution.getSolutionSetHighRating(solutionSet) + "," + Solution.getSolutionSetMeanRating(solutionSet));
            }
            
        } // end loop through generations
        return solutionSet;
    }
    
    
    /** 
     * train the simple genetic algorithm method - null trace
     *      - returns a Vector of solutions 
     *            each rule is a Vector of attribute values plus a conclusion plus a strength of the solution
     *               (the attribute values can include a wildcard)
     *
     */
    public static Vector Train (Vector trainingData, Vector attrDefs, int popSize, int generations, double pMutate, ProblemSpecificInfo probInfo ) {
        // 
        Vector solutionSet = SGADesignProb.Train(attrDefs, popSize, generations, pMutate, probInfo, null);
        return solutionSet;   
    }
    
    
    
}