/*
 * Solution.java
 *
 * Created on July 14, 2005, 11:22 AM
 */
/*
 * Solution.java
 *     used for solutions to problems that are not classification problems - e.g. design, scheduling
 *      Plan on using this for PowerPlant Maint Scheduling and for Student Group assignment
 *
 * Created on July 6, 2005, 1:20 PM
 */

package sgapowerunit;


import java.util.Random;
import java.util.Vector;
import java.text.DecimalFormat;
import datamining.*;

/**
 *
 * @author Mike
 */
public class Solution {
    
    private String name = "unknown";
    
    private String[] pattern;
        
    private double strength = 0;
    
    ///////////////////////////////////////////
    // statics
    ///////////////////////////////////////////
    
    
    // use to generate Solution names
    private static int lastUsed = 0;
        
    // for if user wants to use time as a seed
    private static Random generator = new Random();
    
    protected static double highestRating = -1;
    
    protected static Solution highestRatedSolution = new Solution();
    
    public static DecimalFormat twoDecPlacesFormat = new DecimalFormat ("###,###,##0.00"); 
    
    /** Creates a new instance of Solution */
    public Solution() {
        lastUsed++;
        name = "S" + lastUsed;
        // who knows how long the array should be
        pattern = new String[100];
            }
    
    /**
     * used when creating a solution that is not really ready to be initialized - values will come later
     * - used so name of solution won't be assigned
     */
    public Solution (String dummy) {
        name = "dummy";
        
    }
    
    /** 
     * creates a new solution - given a name and an array of Strings to use for the pattern
     */
    public Solution (String nam, String[] patt) {
        name = nam;
        pattern = patt;
        // take the default (0) strengtrh
    }
    
    /** 
     * creates a new instance of Solution - given attributes but - no choices as to which values to use - use first value for each
     */
    public Solution (Attribute[] attributes) {
        lastUsed++;
        name = "S" + lastUsed;
        pattern = new String[attributes.length];  
        for (int at = 0; at < attributes.length; at++) {
            String curr = attributes[at].getNthPossVal(0);  // use first possible value
            pattern[at] = curr;
        }
    }
    
    /** 
     * creates a new instance of Solution - given Vector of attributes but - no choices as to which values to use - use first value for each
     */
    public Solution (Vector attributes) {
        lastUsed++;
        name = "S" + lastUsed;
        pattern = new String[attributes.size()];  
        for (int at = 0; at < attributes.size(); at++) {
            String curr = ((Attribute)attributes.get(at)).getNthPossVal(0);  // use first possible value
            pattern[at] = curr;
        }
    }
    
    /**
     * create a new instance of Solution - with only the number of attributes known
     */
    public Solution (int len) {
        lastUsed++;
        name = "S" + lastUsed;
        pattern = new String[len];
    }
    
    /** 
     * creates a new random instance of Solution - given attributes 
     */
    public Solution (Attribute[] attributes, Random generator) {
        lastUsed++;
        name = "S" + lastUsed;
        pattern = new String[attributes.length];  
        int numPossValues; 
        int rand;
        for (int at = 0; at < attributes.length; at++) {
            String curr;
            numPossValues = attributes[at].getHowManyPossVal(); 
            rand = generator.nextInt(numPossValues);  // integer from 0 to numPossValues - 1
            // fill in random choice
            curr = attributes[at].getNthPossVal(rand);
            pattern[at] = curr;
        }
    }
    
    
    /** 
     * creates a new random instance of rule - given Vector of attributes 
     */
    public Solution (Vector attributes, Random generator) {
        lastUsed++;
        name = "S" + lastUsed;
        pattern = new String[attributes.size()];  
        int numPossValues; 
        int rand;
        for (int at = 0; at < attributes.size(); at++) {
            String curr;
            numPossValues = ((Attribute)attributes.get(at)).getHowManyPossVal(); 
            rand = generator.nextInt(numPossValues);  // integer from 0 to numPossValues - 1
            // fill in random choice
            curr = ((Attribute)attributes.get(at)).getNthPossVal(rand);
            pattern[at] = curr;
        }
    }
    
    /**
     * report name of rule
     */
    public String getName() {
        return name;
    }
    
    /**
     * report strength of rule
     */
    public double getStrength() {
        return strength;
    }
    
    /** 
     * report contents of Nth clause in pattern
     */
    public String getNthPattern (int N) {
        // make sure that the pattern exists
        if (N < this.getLength()) {
            return this.pattern[N];
        }
        else {
            System.out.println("Program error - looking for non-existent solution pattern");
            System.exit(0);
        }
        return "shit";
    }
    
    /** 
     * report contents of pattern
     */
    public String[] getPattern ( ) {
        return this.pattern;
    }
    
    /**
     * report how many attributes
     */
    public int getLength () {
        return pattern.length;
    }
    
    /**
     * change value of a clause in pattern
     */
    public void setNthpattern(int N, String val) {
        this.pattern[N] = val;
    }
    
    /** 
     * change pattern
     */
    public void setPattern (String[] patt) {
        this.pattern = patt;
    }
    
    /**
     * change name of the solution
     */
    public void setName(String nam) {
        name = nam;
    }
    
    /**
     * change strength of the solution
     */
    public void setStrength(double stren) {
        strength = stren;
    }
    
    
    // could use a version to go to an arbitrary stream so could go to a file
    
    
    /** 
     * create a String representation of the solution (for I/O / debugging)
     */
    public String toString() {
        String res = "solution: " + getName() + " { ";
        for (int i= 0; i < pattern.length; i++) {
            res += " " + pattern[i] + " ";
        }
        res += " } ";
        res += " (strength: " + twoDecPlacesFormat.format(strength) + ")";
        return res;
    }
    
    
    
    
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Static service methods
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     
    
    /** 
     * determine how many possible solutions there could be.
     * Number of possibilities for each attribute multiplied together
     */
    public static int howManyPossibleSolutions(Vector attributes) {
        int poss = 1;
        int numPossValues;
        for (int at = 0; at < attributes.size(); at++) {
            String curr;
            numPossValues = ((Attribute)attributes.get(at)).getHowManyPossVal(); 
            poss = poss * numPossValues;
        }
        return poss;
    }
    
    
    /** 
     * static method to generate a population of solutions randomly (use time as a seed
     */
    public static Solution[] genNRandomSolutions(Attribute attributes[], int N) {
        // use the static version
        // use the other more general method
        Solution [] set = genNRandomSolutions(attributes, generator, N);
        return set;
    }
    
    
    /** 
     * static method to generate a population of solutions randomly (use passed generator)
     */
    public static Solution[] genNRandomSolutions(Attribute attributes[], Random gener, int N) {
        Solution [] set = new Solution[N];
        for (int sol = 0; sol < N; sol++) {
            set[sol] = new Solution(attributes, gener);
        }
        return set;        
    }
    
    
    
    /** 
     * static method to generate a population of solutions randomly (use time as a seed) using Vector of attributes
     */
    public static Solution[] genNRandomSolutions(Vector attributes, int N) {
        // use the static one
        // use the other more general method
        Solution [] set = genNRandomSolutions(attributes, generator, N);
        return set;
    }
    
    
    /** 
     * static method to generate a population of solutions randomly (use passed generator) using Vector of attributes
     */
    public static Solution[] genNRandomSolutions(Vector attributes, Random gener, int N) {
        Solution [] set = new Solution[N];
        for (int sol = 0; sol < N; sol++) {
            set[sol] = new Solution(attributes, gener);
        }
        return set;        
    }
    
    /** 
     * convert an array of solutions into a Vector
     */
    public static Vector convertArrayToVector (Solution[] set) {
        Vector res = new Vector();
        for (int sol = 0; sol < set.length; sol++) {
            res.add(set[sol]);
        }
        return res;
    }
    
    
    /**
     * static method to show all Solutions in a set
     */
    public static void showSolutionSet (Solution[] set) {
        for (int sol = 0; sol < set.length; sol++) {
            System.out.println(set[sol]);
        }
    }
    
    
    /**
     * static method to show all Solutions in a set (Vector)
     */
    public static void showSolutionSet (Vector set) {
        for (int sol = 0; sol < set.size(); sol++) {
            System.out.println(set.get(sol));
        }
    }
    
    /**
     * determine average rating in Solution set
     * Written 07/05/05
     * Assumes rule set isn't empty
     */
    public static double getSolutionSetMeanRating (Solution[] set) {
        double total = 0;
        double mean = 0;
        for (int sol = 0; sol < set.length; sol++) {
            total += set[sol].getStrength();
        }
        mean = total / set.length;
        return mean;
    }
    
    /**
     * determine average rating in Solution set
     * Written 07/05/05
     * Assumes rule set isn't empty
     */
    public static double getSolutionSetMeanRating (Vector set) {
        double total = 0;
        double mean = 0;
        for (int sol = 0; sol < set.size(); sol++) {
            total += ((Solution)set.get(sol)).getStrength();
        }
        mean = total / set.size();
        return mean;
    }
    
    
    /**
     * determine highest rating in Solution set
     * Written 07/05/05
     * Assumes rule set isn't empty
     */
    public static double getSolutionSetHighRating (Solution[] set) {
        double high = -999;
        for (int sol = 0; sol < set.length; sol++) {
            if (set[sol].getStrength() > high) {
                high = set[sol].getStrength();
            }
        }
        return high;
    }
    
    /**
     * determine highest rating in Solution set
     * Written 07/05/05
     * Assumes rule set isn't empty
     */
    public static double getSolutionSetHighRating (Vector set) {
        double high = -999;
        for (int sol = 0; sol < set.size(); sol++) {
            if (((Solution)set.get(sol)).getStrength() > high) {
                high = ((Solution)set.get(sol)).getStrength();
            }
        }
        return high;
    }
    
    
    /**
     * determine lowest rating in Solution set
     * Written 07/07/05
     * Assumes rule set isn't empty
     */
    public static double getSolutionSetLowRating (Solution[] set) {
        double low = 9999;
        for (int sol = 0; sol < set.length; sol++) {
            if (set[sol].getStrength() < low) {
                low = set[sol].getStrength();
            }
        }
        return low;
    }
    
    /**
     * determine lowest rating in Solution set
     * Written 07/07/05
     * Assumes rule set isn't empty
     */
    public static double getSolutionSetLowRating (Vector set) {
        double low = 9999;
        for (int sol = 0; sol < set.size(); sol++) {
            if (((Solution)set.get(sol)).getStrength() < low) {
                low = ((Solution)set.get(sol)).getStrength();
            }
        }
        return low;
    }
    
    /** 
     * perform crossover, producing 2 children
     * if crossOver is 0, children will match the parents, otherwise, cross over before position "crossOver"
     */
    public static void crossover (Solution parent1, Solution parent2, int crossOver, Solution child1, Solution child2) {
        int len = parent1.getLength();
        for (int pos = 0; pos < crossOver; pos++) {
            child1.pattern[pos] = parent2.pattern[pos];
            child2.pattern[pos] = parent1.pattern[pos];
        }
        for (int pos = crossOver; pos < len; pos++) {
            child1.pattern[pos] = parent1.pattern[pos];
            child2.pattern[pos] = parent2.pattern[pos];
        }
        System.out.println("Crossover point: " + crossOver);
        System.out.println("Children");
        System.out.println("     " + child1);
        System.out.println("     " + child2);
    }
        
         
    /**
     * make available random number generator so can be used by others
     */
    public static Random getGenerator () {
        return generator;
    }
    
    /** 
     * (weighted) randomly choose a solution from a solution set (to become a parent) 
     * The chance of a solution being selected is proportional to the strength of the solution
     * uses time as a seed
     */
    public static Solution chooseProportional (Vector solutionSet, double totalRating) {
        // use the static one
        Solution choice = Solution.chooseProportional(solutionSet,totalRating,generator);
        return choice;
    }
    
    
    
    /** 
     * (weighted) randomly choose a solution from a solution set (to become a parent) 
     * The chance of a solution being selected is proportional to the strength of the solution
     * passed a random generator
     */
    public static Solution chooseProportional (Vector solutionSet, double totalRating, Random gener) {
        double rand = gener.nextDouble();  // returns between 0 and 1
        // the larger the number generated the later the rule selected
        //  if 4 rules existed and had strengths 8 6 10 6 then the total would be 30
        //  the rules would get  0.00 to 0.267 (8/30) .267 to .467 (14/30) .467 to .800 (24/30) and .800 up
        //System.out.print("          chances: ");
        // have to be careful - if anybody rated below zero we need to adjust to avoid abnormalities
        double lowest = Solution.getSolutionSetLowRating(solutionSet);
        double accum = 0.00;
        int index = 0;
        boolean found = false;
        while ((!found) && (index < solutionSet.size())) {
            if (lowest < 0) {
                // at least one rating is negative - will subtract (lowest - 5) from each rating - making all ratings positive.
                //    This also means we need an adjusted total
                double adjustedTotal = totalRating + (-1 * (lowest-5) * solutionSet.size()); 
                double adjustedRating = ((Solution)solutionSet.get(index)).getStrength() - (lowest - 5);
                double ratio = adjustedRating / adjustedTotal;
        
                //System.out.print("  | " + twoDecPlacesFormat.format(ratio));
                accum += ratio;
                
            }
            else {
                // normal case
                double ratio = ((Solution)solutionSet.get(index)).getStrength() / totalRating;
                //System.out.print("  | " + twoDecPlacesFormat.format(ratio));
                accum += ratio;
            }
            //System.out.println("      debug - index: " + index + " accum: " + accum + " rand: " + rand);
            if (rand <= accum) {
                found = true;
            }
            else {
                index++;
            }
        }            
        //System.out.println("");
        Solution choice = (Solution)solutionSet.get(index);
        return choice;
    }
}
